library vibe_one;

import 'dart:async';

import 'package:flutter/material.dart';
import 'package:flutter_vibrate/flutter_vibrate.dart';
import 'package:vibration/vibration.dart';

import 'data/db/db.dart';
import 'data/models/mode.dart';

/// A Calculator.
class VibeOneSingleton {
  static bool isPremium = false;

  static VibeOneSingleton? _instance;
  VibeOneSingleton._internal();
  factory VibeOneSingleton({bool isPremium = false}) {
    // ignore: prefer_conditional_assignment
    if (_instance == null) {
      VibeOneSingleton.isPremium = isPremium;
      _instance = VibeOneSingleton._internal();
    }

    return _instance!;
  }

  bool power = false;
  double velocity = 2.0;
  Mode mode = modes[1];
  Timer? _timer;

  setPower(bool value) async {
    power = value;

    if (value) {
      initVibration();
    } else {
      stopVibration();
    }
  }

  setVelocity(double value) async {
    if (_instance == null) return; // ignore: avoid_returning_null

    velocity = value;
    if (power) initVibration();
    return;
  }

  setMode(int value) async {
    mode = modes[value];
    if (power) initVibration();
  }

  initVibration() async {
    final totalTime = getTotalTime(mode.patterns['pauses']);
    final repeatTimer = totalTime ~/ velocity;

    debugPrint("initVibration $repeatTimer");
    if (_timer == null) {
      _timer = Timer.periodic(
        Duration(milliseconds: repeatTimer),
        (Timer t) async {
          logicaVibracao();
        },
      );
    } else {
      stopVibration();
      _timer = Timer.periodic(
        Duration(milliseconds: repeatTimer),
        (Timer t) async => logicaVibracao(),
      );
    }

    logicaVibracao();
  }

  stopVibration() {
    debugPrint('estado Timer: ${_timer!.isActive}');
    if (_timer!.isActive) _timer!.cancel();
  }

  logicaVibracao() async {
    final totalTime = getTotalTime(mode.patterns['pauses']) * 2;
    for (dynamic excLine in mode.patterns['pauses']) {
      debugPrint("excLine: ${DateTime.now()} totalTime: $totalTime");
      if (_timer != null && !_timer!.isActive) break;
      debugPrint("executou");

      //final delay = totalTime ~/ velocity;

      switch (excLine['type']) {
        case 2:
          Vibrate.feedback(excLine['action']);
          await Future.delayed(Duration(milliseconds: excLine['delay'] ~/ velocity));
          await Future.delayed(const Duration(milliseconds: 125));
          break;
        default:
          final vibrateCustom = await Vibration.hasCustomVibrationsSupport();
          if (vibrateCustom!) {
            final isAmplitude = await Vibration.hasAmplitudeControl();
            await Vibration.vibrate(
              duration: excLine['action'],
              amplitude: isAmplitude != null && isAmplitude ? 128 : -1,
              intensities: [128, 255, 128, 255],
            );
            //Vibrate.vibrate();
            await Future.delayed(Duration(milliseconds: excLine['action']));
            await Future.delayed(const Duration(milliseconds: 250));
          } else {
            debugPrint("Vibration.vibrate()");
            await Vibration.vibrate();
            // await Future.delayed(Duration(milliseconds: excLine['action']));
            // await Future.delayed(Duration(milliseconds: delay));
          }
      }
    }
  }

  int getTotalTime(List<dynamic> list) => list.map((e) => e['type'] == 1 ? e['action'] : e['delay']).reduce((value, element) => value + element);
}
